#! /bin/bash
# $Id: char_convert.bash,v 1.1 2004/09/20 16:40:14 mszick Exp $

:<<-'__char_convert_Doc_'

    Copyright (c) Michael S. Zick, 2004
    License: Unrestricted reuse in any form, for any purpose.
    Warranty: None -{Its a script; the user is on their own.}-

Provides
    Conversion (replacement) of characters in a string.

Notes
    If may not be pretty, but Bash does not require
    external programs to do selected character
    replacements in a string.

    Upper case to Lower case demonstrated.

__char_convert_Doc_

# A test string and place for converted string
declare in_str='ABCDABMSZ'
declare out_str

# Replace all occurances of each character
out_str=${in_str//A/a}
out_str=${out_str//B/b}
out_str=${out_str//C/c}

# Display the result
echo
echo 'Test 1:'
echo 'Input string: '${in_str}
echo 'Output string: '${out_str}

# The big, the slow, and the ugly;
# all in one place.
# to_lower <string>
to_lower() {
    local _tl_out
    [ $# -eq 1 ] || return 1
    _tl_out=${1//A/a}
    _tl_out=${_tl_out//B/b}
    _tl_out=${_tl_out//C/c}
    _tl_out=${_tl_out//D/d}
    _tl_out=${_tl_out//E/e}
    _tl_out=${_tl_out//F/f}
    _tl_out=${_tl_out//G/g}
    _tl_out=${_tl_out//H/h}
    _tl_out=${_tl_out//I/i}
    _tl_out=${_tl_out//J/j}
    _tl_out=${_tl_out//K/k}
    _tl_out=${_tl_out//L/l}
    _tl_out=${_tl_out//M/m}
    _tl_out=${_tl_out//N/n}
    _tl_out=${_tl_out//O/o}
    _tl_out=${_tl_out//P/p}
    _tl_out=${_tl_out//Q/q}
    _tl_out=${_tl_out//R/r}
    _tl_out=${_tl_out//S/s}
    _tl_out=${_tl_out//T/t}
    _tl_out=${_tl_out//U/u}
    _tl_out=${_tl_out//V/v}
    _tl_out=${_tl_out//W/w}
    _tl_out=${_tl_out//X/x}
    _tl_out=${_tl_out//Y/y}
    _tl_out=${_tl_out//Z/z}
    echo ${_tl_out}
    return 0
}

# Try our new to_lower function
# String output is on stdout
echo
echo 'Test 2:'

if out_str=$(to_lower ${in_str})
then
    # Display the result
    echo 'Input string: '${in_str}
    echo 'Output string: '${out_str}
else
    echo 'to_lower function failed'
fi

# String output is on stdout
# No argument (or anything that expands to blank)
# should be detected as an error

# NOTE:
# 'nothing' is not define and set in this script.
echo
echo 'Test 3:'

if out_str=$(to_lower ${nothing})
then
    # Display the result
    echo 'Input string: '${in_str}
    echo 'Output string: '${out_str}
else
    echo 'to_lower function failed'
fi
echo

# Why?
# The 'let' command sets and returns an error code.
# The unadorned '=' assignment does not modify the
# error code.
# So the 'if' follows the rule: 'tests last error
# code set'.  In this case, that set by 'to_lower'.

:<<-'_to_lower_outputs_'

Test 1:
Input string: ABCDABMSZ
Output string: abcDabMSZ

Test 2:
Input string: ABCDABMSZ
Output string: abcdabmsz

Test 3:
to_lower function failed

_to_lower_outputs_
exit 0
