#! /bin/bash
# $Id: file_to_array.bash,v 1.5 2004/09/14 22:38:57 mszick Exp $

:<<-'__file_to_array_Doc_'

    Copyright (c) Michael S. Zick, 2004
    License: Unrestricted reuse in any form, for any purpose.
    Warranty: None -{Its a script; the user is on their own.}-

Provides
    Given the name of a file and the name of an array;
    Read the file into the array, one line per element.

__file_to_array_Doc_

# Special IFS settings used for parsing.

# Whitespace == :Space:Tab:Line Feed:Carriage Return:
WSP_IFS=$'\x20'$'\x09'$'\x0A'$'\x0D'

# No Whitespace == Line Feed:Carriage Return
NO_WSP=$'\x0A'$'\x0D'

# Using a special setting of the IFS;
# parse the result of 'cat filename' into
# array elements by assignment.

# It is reasonable to expect this to fail for
# very large files - but known to work for files
# of several thousand lines.

# Which certainly should handle any reasonable
# configuration file.

# file_to_array <file_name> <line_array_name>
file_to_array() {
    [ $# -eq 2 ] || return 1  # two arguments required

    # Make the element separators common line endings.
    local IFS=${NO_WSP}

    # A temporary place to store the lines.
    local -a _fta_tmp_

    # Now just 'do-it' without any 'read until' loops
    _fta_tmp_=( $(cat $1) )

    # Copy-by-Name the return
    eval $2=\( \$\{_fta_tmp_\[@\]\} \)

    # The special IFS setting was local, so it is
    # just dropped when the function returns.
    return 0
}

#### Test Code

if [ $# -ne 1 ]
then
    echo
    echo 'The script >'$0'< requires one argument.'
    echo 'That argument should be the name of a'
    echo 'file that it is allowed to read.'
    echo
    exit 1
fi

declare -a lines
echo
echo "Using file: "$1
echo

file_to_array $1 lines

echo "Element count "${#lines[@]}
ln_cnt=${#lines[@]}

echo
for (( i = 0; i < ${ln_cnt}; i++ ))
do
    echo 'Element '$i': >'${lines[$i]}'<'
done

# Q: What happens to any blank lines in the file?
# Q: Why?

:<<-'__optional_answers'

A1: Blank lines in file are suppressed.

A2.1: I left IFS set at 'no whitespace' during copy.
A2.2: The array assignment form: dest=( ${src[@]} )
      does sub-script packing.

__optional_answers

exit 0