#! /bin/bash
# $Id: is_address.bash,v 1.7 2004/09/23 17:32:25 mszick Exp $

:<<-'__is_address_Doc_'

    Copyright (c) Michael S. Zick, 2004
    License: Unrestricted reuse in any form, for any purpose.
    Warranty: None -{Its a script; the user is on their own.}-

Provides
    Test a string for being a valid numeric IP(v4) address.

__is_address_Doc_


# Special IFS settings used for string parsing.
# Whitespace == :Space:Tab:Line Feed:Carriage Return:
WSP_IFS=$'\x20'$'\x09'$'\x0A'$'\x0D'

# No Whitespace == Line Feed:Carriage Return
NO_WSP=$'\x0A'$'\x0D'

# Field separator for dotted decimal IP addresses
ADR_IFS=${NO_WSP}'.'

# This function described in: is_number.bash
# is_number <input>
is_number() {
    [ $# -eq 1 ]      || return 1   # is blank?
    [ x"$1" == 'x0' ] && return 0   # is zero?
    local -i tst
    let tst=$1 2>/dev/null          # is numeric?
    return $?
}

# Parse by assignment a string and then
# test if a valid numeric IPv4 address.

# is_address <input>
is_address() {
    [ $# -eq 1 ] || return 1    # Blank ==> false

    # An array to hold the fields
    local -a _ia_input

    # Set the local IFS to replace dots with spaces
    local IFS=${ADR_IFS}

    # Parse by assignment
    _ia_input=( $1 )

    # Exhaustive testing of an IPv4 address string

    # If four fields ;
    # and each are numeric;
    # and each less than 256...

    if  [ ${#_ia_input[@]} -eq 4 ]  &&
        is_number ${_ia_input[0]}   &&
        is_number ${_ia_input[1]}   &&
        is_number ${_ia_input[2]}   &&
        is_number ${_ia_input[3]}   &&
        [ ${_ia_input[0]} -lt 256 ] &&
        [ ${_ia_input[1]} -lt 256 ] &&
        [ ${_ia_input[2]} -lt 256 ] &&
        [ ${_ia_input[3]} -lt 256 ]
    then
        return 0
    else
        return 1
    fi
}

echo
# Try passing a domain name to the test function
if is_address really.bad.advice.con
then
    echo '"really.bad.advice.con" is a valid IP address'
else
    echo '"really.bad.advice.con" is NOT a valid IP address'
fi
echo

# Try passing a numeric IP address
if is_address 127.0.127.1
then
    echo '"127.0.127.1" is a valid IP address'
else
    echo '"127.0.127.1" is NOT a valid IP address'
fi
echo

# Try passing a query fragment (note trailing dot)
if is_address 127.0.127.1.
then
    echo '"127.0.127.1." is a valid IP address'
else
    echo '"127.0.127.1." is NOT a valid IP address'
fi
echo

:<<-'_is_address_outputs_'

"really.bad.advice.con" is NOT a valid IP address

"127.0.127.1" is a valid IP address

"127.0.127.1." is a valid IP address

_is_address_outputs_
exit 0