#! /bin/bash
# $Id: is_number.bash,v 1.4 2004/09/23 14:15:18 mszick Exp $

:<<-'__is_number_Doc_'

    Copyright (c) Michael S. Zick, 2004
    License: Unrestricted reuse in any form, for any purpose.
    Warranty: None -{Its a script; the user is on their own.}-

Provides
    Test if a variable is a valid number using assignment.

__is_number_Doc_

# Bash allows variables to be typed.
declare -i tst_num

echo
# The 'let' command sets the return code.
let tst_num=String
echo 'return code from string assignment is: '$?

let tst_num=1234
echo 'return code from number assignment is: '$?

# Since that fails for the value '0'...
# Turn it into a function using a local variable.
# Functions are subshells, so must return $?.

# is_number <input>
is_number() {
    # Argument expanded to blank
    [ "$#" -eq 1 ]    || return 1  # Non-numeric

    # Argument expanded to character '0'
    [ x"$1" == 'x0' ] && return 0  # Numeric

    # Set the type to integer
    local -i tst

    # 'let' acts like an external command
    # send any error messsages to /dev/null
    let tst=$1 2>/dev/null         # is numeric?

    # Functions are sub-shells, return the
    # result code from 'let'
    return $?
}

echo
echo -n 'return code from test given a string: '
is_number String
echo $?
echo
echo -n 'return code from test given a dotted string: '
is_number Dotted.String
echo $?
echo
echo -n 'return code from test given a number is: '
is_number 1234
echo $?
echo

:<<-'_is_number_outputs_'

return code from string assignment is: 1
return code from number assignment is: 0

return code from test given a string: 1

return code from test given a dotted string: 1

return code from test given a number is: 0

_is_number_outputs_
exit 0