#! /bin/bash
# $Id: to_dot.bash,v 1.2 2004/09/23 17:27:34 mszick Exp $

:<<-'__to_dot_Doc_'

    Copyright (c) Michael S. Zick, 2004
    License: Unrestricted reuse in any form, for any purpose.
    Warranty: None -{Its a script; the user is on their own.}-

Provides
    Single character replacement for any one of
    a group of characters.

    Modified string is output on stdout.

__to_dot_Doc_

# Special IFS settings used for string parsing.
# Whitespace == :Space:Tab:Line Feed:Carriage Return:
WSP_IFS=$'\x20'$'\x09'$'\x0A'$'\x0D'

# No Whitespace == Line Feed:Carriage Return
NO_WSP=$'\x0A'$'\x0D'


# Not everybody uses dots as separators (apnic)
# to_dot <string>
to_dot() {
    [ $# -eq 1 ] || return 1

    # local string
    local _td_out

    # replace all occurrences of characters
    # in alternative glob (list) with dot.
    _td_out=${1//[#|@|%]/.}

    # result to stdout
    echo ${_td_out}
    return 0
}

:<<-'_to_dot_alternate_'

to_dot() {
    [ $# -eq 1 ] || return 1
    echo ${1//[#|@|%]/.}
    return 0
}

_to_dot_alternate_

#### Test code ####

declare target
declare base

# Using funny field separators
base='1.2#3@4%5'

# Convert to dot separators
echo
echo 'Test one'
if target=$(to_dot ${base})
then
    # What did that do for us?
    echo '    Input  string >'${base}'<'
    echo '    Output string >'${target}'<'
else
    echo '    Test failed'
fi

# Pass an undefined variable to the function
echo
echo 'Test two'
if target=$(to_dot ${nothing})
then
    # What did that do for us?
    echo '    Input  string >'${nothing}'<'
    echo '    Output string >'${target}'<'
else
    echo '    Test passed'
fi
echo

:<<-'_to_dot_outputs_'

Test one
    Input  string >1.2#3@4%5<
    Output string >1.2.3.4.5<

Test two
    Test passed

_to_dot_outputs_

exit 0